/*
FRODO: a FRamework for Open/Distributed Optimization
Copyright (C) 2008, 2009, 2010, 2011  Thomas Leaute, Brammert Ottens & Radoslaw Szymanek

FRODO is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FRODO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


How to contact the authors: 
<http://lia.epfl.ch/>

EPFL / IC / IIF / LIA
Batiment IN 
Station 14 
CH - 1015 Lausanne 
Switzerland
*/

package ch.epfl.lia.frodo.communication;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectStreamClass;

/** An optimized implementation of ObjectInputStream that minimizes the overhead of serializing class descriptions
 * @author Thomas Leaute
 */
public class ObjectInputStream extends java.io.ObjectInputStream {

	/** Constructor
	 * @param in 			input stream to read from
	 * @throws IOException 	if an I/O error occurs while reading stream header
	 */
	public ObjectInputStream(InputStream in) throws IOException {
		super(in);
	}

	/** @see java.io.ObjectInputStream#readClassDescriptor() */
	protected ObjectStreamClass readClassDescriptor () throws ClassNotFoundException, IOException {
		
		String className = this.readUTF();
		
		// Re-append FRODO's full package name if it has been stripped
		if (className.startsWith("frodo")) 
			className = "ch.epfl.lia." + className;
		
		return ObjectStreamClass.lookup(Class.forName(className));
	}
	
}
